@@ -13,9 +13,8 @@
 
 #include "fakenetwork/settings.h"
 
+#include <QSignalSpy>
 #include <QTest>
-#include <QSignalSpy>
-#include <QSignalSpy>
 
 void SettingsTest::initTestCase()
 {
@@ -24,7 +23,8 @@ void SettingsTest::initTestCase()
 
 void SettingsTest::testConnections()
 {
-    NetworkManager::ConnectionSettings::Ptr connectionSettings = NetworkManager::ConnectionSettings::Ptr(new NetworkManager::ConnectionSettings(NetworkManager::ConnectionSettings::Wired));
+    NetworkManager::ConnectionSettings::Ptr connectionSettings =
+        NetworkManager::ConnectionSettings::Ptr(new NetworkManager::ConnectionSettings(NetworkManager::ConnectionSettings::Wired));
     connectionSettings->setId("Wired connection");
     connectionSettings->setUuid(QLatin1String("39af79a5-b053-4893-9378-7342a5a30d06"));
     NetworkManager::Ipv4Setting::Ptr ipv4Setting = connectionSettings->setting(NetworkManager::Setting::Ipv4).dynamicCast<NetworkManager::Ipv4Setting>();
@@ -84,3 +84,5 @@ void SettingsTest::testConnectionAdded(const QString &connection)
 }
 
 QTEST_MAIN(SettingsTest)
+
+#include "moc_settingstest.cpp"
