@@ -1,34 +1,19 @@
 /*
-    Copyright 2018 Pranav Gade <pranavgade20@gmail.com>
+    SPDX-FileCopyrightText: 2018 Pranav Gade <pranavgade20@gmail.com>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) version 3, or any
-    later version accepted by the membership of KDE e.V. (or its
-    successor approved by the membership of KDE e.V.), which shall
-    act as a proxy defined in Section 6 of version 3 of the license.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 
 #ifndef NETWORKMANAGERQT_PROXY_SETTING_H
 #define NETWORKMANAGERQT_PROXY_SETTING_H
 
-#include <networkmanagerqt/networkmanagerqt_export.h>
 #include "setting.h"
+#include <networkmanagerqt/networkmanagerqt_export.h>
 
 #include <QString>
 
 namespace NetworkManager
 {
-
 class ProxySettingPrivate;
 
 /**
@@ -41,7 +26,7 @@ public:
     typedef QList<Ptr> List;
     enum Mode {
         None = 0,
-        Auto = 1
+        Auto = 1,
     };
 
     ProxySetting();
@@ -78,4 +63,3 @@ NETWORKMANAGERQT_EXPORT QDebug operator<<(QDebug dbg, const ProxySetting &settin
 }
 
 #endif // NETWORKMANAGERQT_PROXY_SETTING_H
-
