@@ -1,32 +1,18 @@
 /*
-    Copyright 2009 Will Stephenson <wstephenson@kde.org>
-    Copyright 2012-2013 Jan Grulich <jgrulich@redhat.com>
+    SPDX-FileCopyrightText: 2009 Will Stephenson <wstephenson@kde.org>
+    SPDX-FileCopyrightText: 2012-2013 Jan Grulich <jgrulich@redhat.com>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) version 3, or any
-    later version accepted by the membership of KDE e.V. (or its
-    successor approved by the membership of KDE e.V.), which shall
-    act as a proxy defined in Section 6 of version 3 of the license.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 
 #include <QRegularExpression>
 
-#include "utils.h"
 #include "time.h"
+#include "utils.h"
 
 QHostAddress NetworkManager::ipv6AddressAsHostAddress(const QByteArray &address)
 {
-//     Q_ASSERT(address.size() == 16);
+    //     Q_ASSERT(address.size() == 16);
     Q_IPV6ADDR tmp;
     for (int i = 0; i < 16; ++i) {
         tmp[i] = address[i];
@@ -39,11 +25,12 @@ QHostAddress NetworkManager::ipv6AddressAsHostAddress(const QByteArray &address)
 
 QByteArray NetworkManager::ipv6AddressFromHostAddress(const QHostAddress &address)
 {
-//     Q_ASSERT(address.protocol() == QAbstractSocket::IPv6Protocol);
+    //     Q_ASSERT(address.protocol() == QAbstractSocket::IPv6Protocol);
     Q_IPV6ADDR tmp = address.toIPv6Address();
     QByteArray assembledAddress;
+    assembledAddress.reserve(16);
     for (int i = 0; i < 16; ++i) {
-        assembledAddress[i] = tmp[i];
+        assembledAddress.push_back(tmp[i]);
     }
 
     return assembledAddress;
@@ -62,14 +49,14 @@ QString NetworkManager::macAddressAsString(const QByteArray &ba)
 
 QByteArray NetworkManager::macAddressFromString(const QString &s)
 {
-    QStringList macStringList = s.split(':');
-//     Q_ASSERT(macStringList.size() == 6);
+    const QStringList macStringList = s.split(':');
+    //     Q_ASSERT(macStringList.size() == 6);
     QByteArray ba;
     if (!s.isEmpty()) {
         ba.resize(6);
         int i = 0;
 
-        Q_FOREACH (const QString & macPart, macStringList) {
+        for (const QString &macPart : macStringList) {
             ba[i++] = macPart.toUInt(nullptr, 16);
         }
     }
@@ -94,7 +81,7 @@ int NetworkManager::findChannel(int freq)
     if (freq < 2500) {
         channel = 0;
         int i = 0;
-        QList<QPair<int, int> > bFreqs = getBFreqs();
+        QList<QPair<int, int>> bFreqs = getBFreqs();
         while (i < bFreqs.size()) {
             if (bFreqs.at(i).second <= freq) {
                 channel = bFreqs.at(i).first;
@@ -107,7 +94,7 @@ int NetworkManager::findChannel(int freq)
     }
     channel = 0;
     int i = 0;
-    QList<QPair<int, int> > aFreqs = getAFreqs();
+    QList<QPair<int, int>> aFreqs = getAFreqs();
     while (i < aFreqs.size()) {
         if (aFreqs.at(i).second <= freq) {
             channel = aFreqs.at(i).first;
@@ -129,7 +116,9 @@ NetworkManager::WirelessSetting::FrequencyBand NetworkManager::findFrequencyBand
     return WirelessSetting::A;
 }
 
-bool NetworkManager::deviceSupportsApCiphers(NetworkManager::WirelessDevice::Capabilities interfaceCaps, NetworkManager::AccessPoint::WpaFlags apCiphers, WirelessSecurityType type)
+bool NetworkManager::deviceSupportsApCiphers(NetworkManager::WirelessDevice::Capabilities interfaceCaps,
+                                             NetworkManager::AccessPoint::WpaFlags apCiphers,
+                                             WirelessSecurityType type)
 {
     bool havePair = false;
     bool haveGroup = true;
@@ -170,21 +159,27 @@ bool NetworkManager::deviceSupportsApCiphers(NetworkManager::WirelessDevice::Cap
 }
 
 // Keep this in sync with NetworkManager/libnm-core/nm-utils.c:nm_utils_security_valid()
-bool NetworkManager::securityIsValid(WirelessSecurityType type, NetworkManager::WirelessDevice::Capabilities interfaceCaps, bool haveAp, bool adhoc, NetworkManager::AccessPoint::Capabilities apCaps, NetworkManager::AccessPoint::WpaFlags apWpa, NetworkManager::AccessPoint::WpaFlags apRsn)
+bool NetworkManager::securityIsValid(WirelessSecurityType type,
+                                     NetworkManager::WirelessDevice::Capabilities interfaceCaps,
+                                     bool haveAp,
+                                     bool adhoc,
+                                     NetworkManager::AccessPoint::Capabilities apCaps,
+                                     NetworkManager::AccessPoint::WpaFlags apWpa,
+                                     NetworkManager::AccessPoint::WpaFlags apRsn)
 {
     bool good = true;
 
-    //kDebug() << "type(" << type << ") interfaceCaps(" << interfaceCaps << ") haveAp(" << haveAp << ") adhoc(" << adhoc << ") apCaps(" << apCaps << ") apWpa(" << apWpa << " apRsn(" << apRsn << ")";
+    // kDebug() << "type(" << type << ") interfaceCaps(" << interfaceCaps << ") haveAp(" << haveAp << ") adhoc(" << adhoc << ") apCaps(" << apCaps << ") apWpa("
+    // << apWpa << " apRsn(" << apRsn << ")";
 
     if (!haveAp) {
         if (type == NoneSecurity) {
             return true;
         }
-        if ((type == StaticWep)
-                || ((type == DynamicWep) && !adhoc)
-                || ((type == Leap) && !adhoc)) {
-            if (interfaceCaps.testFlag(NetworkManager::WirelessDevice::Wep40) ||
-                    interfaceCaps.testFlag(NetworkManager::WirelessDevice::Wep104)) {
+        if ((type == StaticWep) //
+            || ((type == DynamicWep) && !adhoc) //
+            || ((type == Leap) && !adhoc)) {
+            if (interfaceCaps.testFlag(NetworkManager::WirelessDevice::Wep40) || interfaceCaps.testFlag(NetworkManager::WirelessDevice::Wep104)) {
                 return true;
             } else {
                 return false;
@@ -195,20 +190,20 @@ bool NetworkManager::securityIsValid(WirelessSecurityType type, NetworkManager::
         // see libs/internals/wirelessinterfaceconnectionhelpers.cpp
 
         // TODO: this is not in nm-utils.c
-//         if (type == Knm::WirelessSecurity::WpaPsk
-//                 || ((type == Knm::WirelessSecurity::WpaEap) && !adhoc)) {
-//             if (interfaceCaps.testFlag(NetworkManager::WirelessDevice::Wpa) &&
-//                 !apCaps.testFlag(NetworkManager::AccessPoint::Privacy)) {
-//                 return true;
-//             }
-//         }
-//         if (type == Knm::WirelessSecurity::Wpa2Psk
-//                 || ((type == Knm::WirelessSecurity::Wpa2Eap) && !adhoc)) {
-//             if (interfaceCaps.testFlag(NetworkManager::WirelessDevice::Rsn) &&
-//                 !apCaps.testFlag(NetworkManager::AccessPoint::Privacy)) {
-//                 return true;
-//             }
-//         }
+        //         if (type == Knm::WirelessSecurity::WpaPsk
+        //                 || ((type == Knm::WirelessSecurity::WpaEap) && !adhoc)) {
+        //             if (interfaceCaps.testFlag(NetworkManager::WirelessDevice::Wpa) &&
+        //                 !apCaps.testFlag(NetworkManager::AccessPoint::Privacy)) {
+        //                 return true;
+        //             }
+        //         }
+        //         if (type == Knm::WirelessSecurity::Wpa2Psk
+        //                 || ((type == Knm::WirelessSecurity::Wpa2Eap) && !adhoc)) {
+        //             if (interfaceCaps.testFlag(NetworkManager::WirelessDevice::Rsn) &&
+        //                 !apCaps.testFlag(NetworkManager::AccessPoint::Privacy)) {
+        //                 return true;
+        //             }
+        //         }
     }
 
     switch (type) {
@@ -232,10 +227,11 @@ bool NetworkManager::securityIsValid(WirelessSecurityType type, NetworkManager::
             return false;
         }
         if (apWpa || apRsn) {
-            if (!deviceSupportsApCiphers(interfaceCaps, apWpa, StaticWep))
+            if (!deviceSupportsApCiphers(interfaceCaps, apWpa, StaticWep)) {
                 if (!deviceSupportsApCiphers(interfaceCaps, apRsn, StaticWep)) {
                     return false;
                 }
+            }
         }
         break;
     case DynamicWep:
@@ -266,12 +262,12 @@ bool NetworkManager::securityIsValid(WirelessSecurityType type, NetworkManager::
         }
         if (haveAp) {
             if (apWpa.testFlag(NetworkManager::AccessPoint::KeyMgmtPsk)) {
-                if (apWpa.testFlag(NetworkManager::AccessPoint::PairTkip) &&
-                        interfaceCaps.testFlag(NetworkManager::WirelessDevice::Tkip)) {
+                if (apWpa.testFlag(NetworkManager::AccessPoint::PairTkip) //
+                    && interfaceCaps.testFlag(NetworkManager::WirelessDevice::Tkip)) {
                     return true;
                 }
-                if (apWpa.testFlag(NetworkManager::AccessPoint::PairCcmp) &&
-                        interfaceCaps.testFlag(NetworkManager::WirelessDevice::Ccmp)) {
+                if (apWpa.testFlag(NetworkManager::AccessPoint::PairCcmp) //
+                    && interfaceCaps.testFlag(NetworkManager::WirelessDevice::Ccmp)) {
                     return true;
                 }
             }
@@ -287,18 +283,18 @@ bool NetworkManager::securityIsValid(WirelessSecurityType type, NetworkManager::
                 if (!interfaceCaps.testFlag(NetworkManager::WirelessDevice::IBSSRsn)) {
                     return false;
                 }
-                if (apRsn.testFlag(NetworkManager::AccessPoint::PairCcmp) &&
-                        interfaceCaps.testFlag(NetworkManager::WirelessDevice::Ccmp)) {
+                if (apRsn.testFlag(NetworkManager::AccessPoint::PairCcmp) //
+                    && interfaceCaps.testFlag(NetworkManager::WirelessDevice::Ccmp)) {
                     return true;
                 }
             } else {
                 if (apRsn.testFlag(NetworkManager::AccessPoint::KeyMgmtPsk)) {
-                    if (apRsn.testFlag(NetworkManager::AccessPoint::PairTkip) &&
-                            interfaceCaps.testFlag(NetworkManager::WirelessDevice::Tkip)) {
+                    if (apRsn.testFlag(NetworkManager::AccessPoint::PairTkip) //
+                        && interfaceCaps.testFlag(NetworkManager::WirelessDevice::Tkip)) {
                         return true;
                     }
-                    if (apRsn.testFlag(NetworkManager::AccessPoint::PairCcmp) &&
-                            interfaceCaps.testFlag(NetworkManager::WirelessDevice::Ccmp)) {
+                    if (apRsn.testFlag(NetworkManager::AccessPoint::PairCcmp) //
+                        && interfaceCaps.testFlag(NetworkManager::WirelessDevice::Ccmp)) {
                         return true;
                     }
                 }
@@ -349,18 +345,18 @@ bool NetworkManager::securityIsValid(WirelessSecurityType type, NetworkManager::
                 if (!interfaceCaps.testFlag(NetworkManager::WirelessDevice::IBSSRsn)) {
                     return false;
                 }
-                if (apRsn.testFlag(NetworkManager::AccessPoint::PairCcmp) &&
-                        interfaceCaps.testFlag(NetworkManager::WirelessDevice::Ccmp)) {
+                if (apRsn.testFlag(NetworkManager::AccessPoint::PairCcmp) //
+                    && interfaceCaps.testFlag(NetworkManager::WirelessDevice::Ccmp)) {
                     return true;
                 }
             } else {
                 if (apRsn.testFlag(NetworkManager::AccessPoint::KeyMgmtSAE)) {
-                    if (apRsn.testFlag(NetworkManager::AccessPoint::PairTkip) &&
-                            interfaceCaps.testFlag(NetworkManager::WirelessDevice::Tkip)) {
+                    if (apRsn.testFlag(NetworkManager::AccessPoint::PairTkip) //
+                        && interfaceCaps.testFlag(NetworkManager::WirelessDevice::Tkip)) {
                         return true;
                     }
-                    if (apRsn.testFlag(NetworkManager::AccessPoint::PairCcmp) &&
-                            interfaceCaps.testFlag(NetworkManager::WirelessDevice::Ccmp)) {
+                    if (apRsn.testFlag(NetworkManager::AccessPoint::PairCcmp) //
+                        && interfaceCaps.testFlag(NetworkManager::WirelessDevice::Ccmp)) {
                         return true;
                     }
                 }
@@ -368,6 +364,17 @@ bool NetworkManager::securityIsValid(WirelessSecurityType type, NetworkManager::
             return false;
         }
         break;
+    case Wpa3SuiteB192:
+        if (adhoc) {
+            return false;
+        }
+        if (!interfaceCaps.testFlag(NetworkManager::WirelessDevice::Rsn)) {
+            return false;
+        }
+        if (haveAp && !apRsn.testFlag(NetworkManager::AccessPoint::KeyMgmtEapSuiteB192)) {
+            return false;
+        }
+        break;
     default:
         good = false;
         break;
@@ -376,17 +383,29 @@ bool NetworkManager::securityIsValid(WirelessSecurityType type, NetworkManager::
     return good;
 }
 
-NetworkManager::WirelessSecurityType NetworkManager::findBestWirelessSecurity(NetworkManager::WirelessDevice::Capabilities interfaceCaps, bool haveAp, bool adHoc, NetworkManager::AccessPoint::Capabilities apCaps, NetworkManager::AccessPoint::WpaFlags apWpa, NetworkManager::AccessPoint::WpaFlags apRsn)
+NetworkManager::WirelessSecurityType NetworkManager::findBestWirelessSecurity(NetworkManager::WirelessDevice::Capabilities interfaceCaps,
+                                                                              bool haveAp,
+                                                                              bool adHoc,
+                                                                              NetworkManager::AccessPoint::Capabilities apCaps,
+                                                                              NetworkManager::AccessPoint::WpaFlags apWpa,
+                                                                              NetworkManager::AccessPoint::WpaFlags apRsn)
 {
-    QList<NetworkManager::WirelessSecurityType> types;
-
     // The ordering of this list is a pragmatic combination of security level and popularity.
     // Therefore static WEP is before LEAP and Dynamic WEP because there is no way to detect
     // if an AP is capable of Dynamic WEP and showing Dynamic WEP first would confuse
     // Static WEP users.
-    types << NetworkManager::SAE << NetworkManager::Wpa2Eap << NetworkManager::Wpa2Psk << NetworkManager::WpaEap << NetworkManager::WpaPsk << NetworkManager::StaticWep << NetworkManager::DynamicWep << NetworkManager::Leap << NetworkManager::NoneSecurity;
+    const QList<NetworkManager::WirelessSecurityType> types = {NetworkManager::Wpa3SuiteB192,
+                                                               NetworkManager::SAE,
+                                                               NetworkManager::Wpa2Eap,
+                                                               NetworkManager::Wpa2Psk,
+                                                               NetworkManager::WpaEap,
+                                                               NetworkManager::WpaPsk,
+                                                               NetworkManager::StaticWep,
+                                                               NetworkManager::DynamicWep,
+                                                               NetworkManager::Leap,
+                                                               NetworkManager::NoneSecurity};
 
-    Q_FOREACH (NetworkManager::WirelessSecurityType type, types) {
+    for (NetworkManager::WirelessSecurityType type : types) {
         if (NetworkManager::securityIsValid(type, interfaceCaps, haveAp, adHoc, apCaps, apWpa, apRsn)) {
             return type;
         }
@@ -482,14 +501,16 @@ NetworkManager::WirelessSecurityType NetworkManager::securityTypeFromConnectionS
         return Wpa2Eap;
     } else if (wifiSecuritySetting->keyMgmt() == WirelessSecuritySetting::SAE) {
         return SAE;
+    } else if (wifiSecuritySetting->keyMgmt() == WirelessSecuritySetting::WpaEapSuiteB192) {
+        return Wpa3SuiteB192;
     }
 
     return NoneSecurity;
 }
 
-QList<QPair<int, int> > NetworkManager::getBFreqs()
+QList<QPair<int, int>> NetworkManager::getBFreqs()
 {
-    QList<QPair<int, int> > freqs;
+    QList<QPair<int, int>> freqs;
 
     freqs.append(QPair<int, int>(1, 2412));
     freqs.append(QPair<int, int>(2, 2417));
@@ -509,9 +530,9 @@ QList<QPair<int, int> > NetworkManager::getBFreqs()
     return freqs;
 }
 
-QList<QPair<int, int> > NetworkManager::getAFreqs()
+QList<QPair<int, int>> NetworkManager::getAFreqs()
 {
-    QList<QPair<int, int> > freqs;
+    QList<QPair<int, int>> freqs;
 
     freqs.append(QPair<int, int>(7, 5035));
     freqs.append(QPair<int, int>(8, 5040));
@@ -559,8 +580,8 @@ QList<QPair<int, int> > NetworkManager::getAFreqs()
     return freqs;
 }
 
-QDateTime NetworkManager::clockBootTimeToDateTime(qlonglong clockBootime) {
-
+QDateTime NetworkManager::clockBootTimeToDateTime(qlonglong clockBootime)
+{
     clockid_t clk_id = CLOCK_BOOTTIME;
     struct timespec tp;
     int r;
@@ -568,10 +589,10 @@ QDateTime NetworkManager::clockBootTimeToDateTime(qlonglong clockBootime) {
     // now is used as a point of reference
     // with the timespec that contains the number of msec since boot
     QDateTime now = QDateTime::currentDateTime();
-    r = clock_gettime (clk_id, &tp);
+    r = clock_gettime(clk_id, &tp);
     if (r == -1 && errno == EINVAL) {
         clk_id = CLOCK_MONOTONIC;
-        r = clock_gettime (clk_id, &tp);
+        r = clock_gettime(clk_id, &tp);
     }
 
     // convert to msecs
